//
//  AdBridger.swift
//  nend_plugin
//
//

import UIKit
import Flutter

class AdBridger: NSObject {
    let KEY_ERROR_CODE = "errorCode"
    
    let targetName: String
    let mappingId: String
    let channel: FlutterMethodChannel

    var rootViewController: UIViewController {
        return UIApplication.shared.delegate!.window!!.rootViewController!
    }
    
    class func adUnit(from argument: Any?) -> AdUnitCodable {
        let jsonData = try! JSONSerialization.data(withJSONObject: argument!, options: [])
        return try! JSONDecoder().decode(AdUnitCodable.self, from: jsonData)
    }
    
    static func frame(from argument: Any?) -> FrameCodable {
        let jsonData = try! JSONSerialization.data(withJSONObject: argument!, options: [])
        return try! JSONDecoder().decode(FrameCodable.self, from: jsonData)
    }
    
    init(with channel: FlutterMethodChannel, tag: SwiftNendPlugin.ClassNameTag, mappingId: String) {
        self.channel = channel
        self.targetName = tag.rawValue
        self.mappingId = mappingId
    }
    
    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        result(FlutterMethodNotImplemented)
    }
    
    func targetMethod(from: FlutterMethodCall) -> MethodName {
        guard let methodName = MethodName(rawValue: SwiftNendPlugin.targetFunc(method: from.method, className: targetName)) else {
            return MethodName.notFound
        }
        return methodName
    }
    
    func invokeDelegates(event: CallbackName, args: [String: Any]?) {
        channel.invokeMethod(targetName + "." + event.rawValue, arguments: SwiftNendPlugin.mappingArguments(mappingId: mappingId, args: args))
    }
    
    enum MethodName: String {
        case loadAd,
        releaseAd,
        showAd,
        hideAd,
        layout,
        //どの広告だけでも定義されているのは↑もにまとめとく
        
        resume,
        pause,
        
        isReady,
        mediationName,
        userId,
        userFeature,
        locationEnabled,
        addFallbackFullboard,
        isEnableAutoReload,
        dismissAd,
        muteStartPlaying,
        //特定の広告だけで定義されているのは↑な感じでまとめとく
        
        //ここより↑に追加していく感じでm(_ _)m
        notFound
    }

    enum CallbackName: String {
        case onLoaded,
        onFailedToLoad,
        onAdClicked,
        onInformationClicked,
        //どの広告だけでも定義されているのは↑もにまとめとく
        
        onFailedToPlay,
        onShown,
        onClosed,
        onStarted,
        onStopped,
        onCompleted,
        onRewarded,
        onFailedToShow,
        //特定の広告だけで定義されているのは↑な感じでまとめとく
        
        //ここより↑に追加していく感じでm(_ _)m
        notFound
    }
}
