
import 'package:flutter/services.dart';
import 'nend_plugin.dart';
import 'video.dart';

class RewardedVideo extends Video {
    static const tag = 'NendAdRewardedVideo';

    static RewardedVideo _shared;
    factory RewardedVideo(int spotId, String apiKey) {
        if (_shared == null) {
            _shared = new RewardedVideo._internal(tag, spotId, apiKey);
        }
        return _shared;
    }
    RewardedVideo._internal(tag, spotId, apiKey) : super(tag, spotId, apiKey);

    @override void releaseAd() async {
        super.releaseAd();
        _shared = null;
    }

    RewardedVideoAdListener _listener;
    RewardedVideoAdListener get listener => _listener;
    set listener(RewardedVideoAdListener value) {
        _listener = value;
        NendPlugin.addListener(tag, this);
    }

    dynamic eventFrom(String type) {
        type = type.split(".").last;
        type = 'RewardedVideoAdEvent.$type';
        return RewardedVideoAdEvent.values.firstWhere((event)=> event.toString() == type, orElse: () => null);
    }

    void notify(MethodCall call) {
        RewardedVideoAdEvent event = eventFrom(call.method);
        if (event == RewardedVideoAdEvent.onFailedToLoad) {
            Map<String, dynamic> map = call.arguments;
            int code = map[NendPlugin.key_error_code];
            listener(event, errorCode: code);
        } else if (event == RewardedVideoAdEvent.onRewarded) {
            Map<String, dynamic> map = call.arguments;
            Map<String, dynamic> reward = map['reward'];
            listener(event, item: RewardedItem()
                ..currencyName = reward['currencyName']
                ..currencyAmount = reward['currencyAmount']
            );
        } else {
            listener(event);
        }
    }
}

enum RewardedVideoAdEvent {
    onLoaded,
    onFailedToLoad,
    onFailedToPlay,
    onShown,
    onStarted,
    onStopped,
    onCompleted,
    onInformationClicked,
    onAdClicked,
    onClosed,
    onRewarded
}

typedef void RewardedVideoAdListener(RewardedVideoAdEvent event, {int errorCode, RewardedItem item});

class RewardedItem {
    int currencyAmount;
    String currencyName;
}