import 'dart:async';

import 'package:flutter/services.dart';
import 'nend_plugin.dart';
import 'user_feature.dart';

abstract class Video {
    String _targetClassName;
    static MethodChannel _channel = NendPlugin.channel;
    Map simpleMappingArgument;

    Video(String name, int spotId, String apiKey) {
        _targetClassName = name;
        invoke(NendPlugin.method_name_init, argument: adUnit(spotId, apiKey));
        simpleMappingArgument = {NendPlugin.key_mapping_id: _targetClassName};
    }

    Map mappingId({Map argument}) {
        if (argument == null) {
            return simpleMappingArgument;
        } else {
            argument[NendPlugin.key_mapping_id] = _targetClassName;
        }
        return argument;
    }

    dynamic adUnit(int spotId, String apiKey) {
        return {
            NendPlugin.key_mapping_id: _targetClassName,
            NendPlugin.key_ad_unit: {NendPlugin.key_spot_id: spotId, NendPlugin.key_api_key: apiKey}
        };
    }

    Future<bool> get isReady async {
        return await _channel.invokeMethod(_targetClassName + '.isReady', mappingId());
    }

    void loadAd() {
        invoke(NendPlugin.method_name_load_ad);
    }

    void showAd() {
        invoke(NendPlugin.method_name_show_ad);
    }

    void releaseAd() async {
        await invoke(NendPlugin.method_name_release_ad);
        await invoke(NendPlugin.method_name_dispose);
    }

    set mediationName(String value) {
        invoke('.mediationName', argument: {'mediationName': value});
    }

    set userId(String value) {
        invoke('.userId', argument: {'userId': value});
    }

    set userFeature(UserFeature feature) {
        invoke('.userFeature', argument: {'userFeature': {
            'gender': feature.gender.index,
            'age': feature.age,
            'birthday': feature.birthday,
            'customStringParams': feature.customStringParams,
            'customIntegerParams': feature.customIntegerParams,
            'customDoubleParams': feature.customDoubleParams,
            'customBooleanParams': feature.customBooleanParams
        }});
    }

    set locationEnabled(bool enabledLocation) {
        invoke('.locationEnabled', argument:  {'locationEnabled': enabledLocation});
    }

    Future<void> invoke(String method, {Map argument}) async {
        argument = mappingId(argument: argument);
        await _channel.invokeMethod(_targetClassName + method, argument);
    }

    dynamic eventFrom(String method);
    void notify(MethodCall call);
}
